// **********************************************************************
//
// Copyright (c) 2003-2010 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <Ice/Ice.h>
#include <Hello.h>

using namespace std;
using namespace Demo;

class CICECallback: public IceUtil::Shared
{
public:
	void sayhelloICECallback()
	{
	}

	void failureICECallback(const Ice::Exception& ex)
	{
	}
};

typedef IceUtil::Handle<CICECallback> CICECallbackPtr;

int
main(int argc, char* argv[])
{
    Ice::CommunicatorPtr communicator;

    try
    {
        communicator = Ice::initialize(argc, argv);

	CICECallbackPtr callbackObj=new CICECallback();
	Callback_Hello_sayHelloPtr cb = newCallback_Hello_sayHello(callbackObj,&CICECallback::sayhelloICECallback, &CICECallback::failureICECallback);

	//HelloPrx hello = HelloPrx::checkedCast(communicator->stringToProxy("hello:tcp -p 10001")->ice_timeout( 200 )); <- if proxy created here segfault disappears 
	int max=80;

	for(int l=0;l<max;l++)
	{
		int timeout=(max-l)*5;

		cout << "loop: "<<(l+1)<<"/"<<max<< " timeout: "<<timeout<<"ms"<<endl;
		try
		{
			HelloPrx hello = HelloPrx::checkedCast(communicator->stringToProxy("hello:tcp -p 10000")->ice_timeout( timeout ));

			for(int j=0;j<20000;j++)
			{				
				hello->begin_sayHello(cb);
			}			
		}
		catch(Ice::ConnectTimeoutException& cex)
		{
			cerr << cex << endl;
		}
		catch(Ice::TimeoutException& tex)
		{
			cerr << tex << endl;
		}
	
	}
        communicator->destroy();
    }
    catch(const Ice::Exception& ex)
    {
        cerr << ex << endl;
        try
        {
            if(communicator)
            {
                communicator->destroy();
            }
        }
        catch(const Ice::Exception& ex)
        {
            cerr << ex << endl;
        }
        exit(1);
    }
    return 0;
}
